#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include "ts/opt.c"
#include "ts/nosig.h"
#include "ts/Array.h"

typedef unsigned short uint16;

union semun {
  int              val;    /* Value for SETVAL */
  struct semid_ds *buf;    /* Buffer for IPC_STAT, IPC_SET */
  unsigned short  *array;  /* Array for GETALL, SETALL */
  struct seminfo  *__buf;  /* Buffer for IPC_INFO
			      (Linux-specific) */
};

ArrayAutoOfSize(int,Lock,0);
ArrayAutoOfSize(int,Unlock,0);
ArrayAutoOfSize(int,WaitLock,0);

int OptionQueue(char *arg,void *target0,int opt) {
  int **target = target0;

  ArrayQueue(target,A(int,strtoul(arg,0,0)));
  return 1;
}

int SemaphoresDelete(int key) {
  int semid,r;

  semid = semget(key, 0, 0777);
  if (semid == -1) {
    printf("Semaphore already does not exist\n");
    return 0;
  }
  if (semctl(semid,0,IPC_RMID,0) == -1) {
    return errno;
  } else {
    return 0;
  }
}

int SemaphoresGet(int key,int count) {
  struct sembuf sop[count];
  int i,semid;
  uint16 arr[count];
  union semun se;

  for (i=0;i<count;i++) arr[i] = 1;
  se.array=arr;
  semid = semget(key, count, IPC_CREAT|IPC_EXCL|0777);
  if (semid != -1) {
    //printf("Creating semaphore\n");
    semctl(semid,0,SETALL,se);
    /*
    for (i=0;i<count;i++) {
      sop[i].sem_num = i;
      sop[i].sem_op = 1;
      sop[i].sem_flg = 0;
    }
    if (TEMP_FAILURE_RETRY(semop(semid, sop, count)) != 0) {
      return -1000+errno; 
    }
    */
  } else {
    //printf("Semaphore exists %m %d\n",count);
    semid = semget(key, 0, 0777);
  }
  if (semid == -1) { 
    return -1100+errno;
  }
  return semid;
}

uint16* SemaphoresQuery(int semid) {
    union semun se;
    struct semid_ds ds;
    int r;
    uint16* arr;

    se.buf = &ds;
    r = semctl(semid,0,IPC_STAT,se);
    if (r < 0) {
      return 0;
    }
    se.array = arr = ArrayAlloc(ds.sem_nsems,2);
    r = semctl(semid, 0, GETALL,se);
    if (r < 0) { 
      ArrayFree(arr);
      return 0;
    }
    return arr;
}

enum {
  SemaphoreLock = -1,
  SemaphoreUnlock = 1,
  SemaphoreWaitLock = 0
};

// flags: IPC_NOWAIT, SEM_UNDO
int SemaphoresOp(int op,int semid,int len,int *num,int flags) {
  struct sembuf sop[len];
  int i,r;

  if (len <= 0) return 0;

  //printf("Op=%d ",op);
  for (i=0;i<len;i++) {
    //printf("%d ",num[i]);
    sop[i].sem_num = num[i];
    sop[i].sem_op = op;
    sop[i].sem_flg = flags;
  }
  //printf("\n");
  r = TEMP_FAILURE_RETRY(semop(semid, sop, len));
  //printf("Done\n");
  if (r != 0) { 
    return -errno;
  }
  return 0;
}

int main(int argc,char *argv[]) {
  int semid,i,len,r;
  int OptMaxFlags=250,OptQuery=0,OptDelete=0,OptNoExit=0;
  uint16* arr;

  key_t semkey = 0x75000000;
  struct option2 opts[] = {
    { 1, OptionUnsigned, &OptMaxFlags, "<f>lags",  "<n> Total number of flags to allocate in the semaphore" },
    { 1, OptionQueue,    &WaitLock,    "<w>ait",   "<n> wait for resource n to be locked(may block)" },
    { 1, OptionQueue,    &Lock,        "<l>ock",   "<n> lock resource n (may block)" },
    { 1, OptionQueue,    &Unlock,      "<u>nlock", "<n> unlock resource n)" },
    { 0, OptionBoolean,  &OptQuery,    "<q>uery",  "display semaphore state before locking or unlocking" },
    { 0, OptionBoolean,  &OptDelete,   "<d>elete", "delete the semaphore and exit" },
    { 0, OptionBoolean,  &OptNoExit,   "<n>oexit", "don't exit after delete" },
    { 0,0,0, "Semaphore Test Program\n", 
      "Query will be done first, then locks, then unlocks, then waits.\n"}
  };

  if (OptionsParse(argc,argv,opts) <= 0) return 0;

  if (OptDelete) {
    SemaphoresDelete(semkey);
    if (!OptNoExit) {
      return 0;
    }
  }

  semid = SemaphoresGet(semkey,OptMaxFlags);

  arr = SemaphoresQuery(semid);
  if (!arr) {
    printf("error:%m\n");
    return 1;
  }
  if (OptQuery) {
    printf("semaphores=%d (%d)\n",r,ArrayLength(arr));
    if (r >= 0) {
      len = ArrayLength(arr);
      printf("Locked: ");
      for (i=0;i<len;i++) {
	printf("%d ",arr[i]);
	//if (arr[i] == 0) printf("%d ",i);
      }
      printf("\n");
    }
  }
  i = SemaphoresOp(SemaphoreLock,semid,ArrayLength(Lock),Lock,0);
  if (i < 0) printf("lockerr=%d\n",i);
  i = SemaphoresOp(SemaphoreUnlock,semid,ArrayLength(Unlock),Unlock,0);
  if (i < 0) printf("unlockerr=%d\n",i);
  i = SemaphoresOp(SemaphoreWaitLock,semid,ArrayLength(WaitLock),WaitLock,0);
  if (i < 0) printf("waiterr=%d\n",i);
}
