#include "generic.h"
// #define type <type of array>
// #define type_null <value to return for empty array>

// Init allocates arr[0] to a zero-length array, unless it already exists
#define Init0 TEMPLATE(Type,Init0)
void Init0(Type **arr) {
  if (arr[0]) return;
  arr[0] = APtr(Type,realloc(MPtr(arr[0]),4));
  ArrayLen(arr[0]) = 0;
}

#define Before TEMPLATE(Type,Before)
void Before(Type **arr,int offset,Type entry) {
  int i,len;
  ArrayOffsetNormalize(arr[0],offset);
  len = ArrayLen0(arr[0]);
  arr[0] = APtr(Type,realloc(MPtr(arr[0]),4+(1+ArrayLen0(arr[0])) * sizeof(Type)));
  ArrayLen(arr[0]) = len+1;
  for (i=ArrayLen(arr[0])-1;i>offset;i--) {
    arr[0][i] = arr[0][i-1];
  }
  arr[0][i] = entry;
}

/*
#define Push TEMPLATE(Type,Push)
static inline void Push(Type **arr,Type entry) {
  Before(arr,0,entry);
}
*/

#define After TEMPLATE(Type,After)
void After(Type **arr,int offset,Type entry) {
  int i,len;
  ArrayOffsetNormalize(arr[0],offset);
  len = ArrayLen0(arr[0]);
  arr[0] = APtr(Type,realloc(MPtr(arr[0]),4+(1+ArrayLen0(arr[0])) * sizeof(Type)));
  ArrayLen(arr[0]) = len+1;
  for (i=ArrayLen(arr[0])-1;i>offset+1;i--) {
    arr[0][i] = arr[0][i-1];
  }
  arr[0][i] = entry;  
}

/*
#define Queue TEMPLATE(Type,Queue)
static inline void Queue(Type **arr,Type entry) {
  After(arr,ArrayLen0(arr[0])-1,entry);
}
*/

#define Delete TEMPLATE(Type,Delete)
Type Delete(Type **arr,int offset,int count) {
  Type ret;
  int i;

  if (count <= 0) count = ArrayLen(arr[0])-offset+count+1;
  if (count <= 0) return (Type)type_null;
  ArrayOffsetNormalize(arr[0],offset);
  if (ArrayLen0(arr[0]) == 0) return (Type)type_null;
  if (offset == ArrayLen(arr[0])) return (Type)type_null;
  if (offset+count > ArrayLen(arr[0])) count = ArrayLen(arr[0])-offset;
  ret = arr[0][offset];
  for (i=offset;i<ArrayLen(arr[0])-count;i++) {
    arr[0][i] = arr[0][i+count];
  }
  ArrayLen(arr[0])-=count;
  arr[0] = APtr(Type,realloc(MPtr(arr[0]),4+(1+ArrayLen0(arr[0])) * sizeof(Type)));
  return ret;
}

/*
#define Pop TEMPLATE(Type,Pop)
inline Type Pop(Type **arr) {
  return Delete(arr,0,1);
}

#define Pull TEMPLATE(Type,Pull)
static inline Type Pull(Type **arr) {
  return Delete(arr,-1,1);
}
*/

// ForEach == Find using a generalize function for Compare
#define FindLinear TEMPLATE(Type,FindLinear)
Type *FindLinear(Type *arr,Type value,int (*Compare)(Type,Type)) {
  int i;
  for (i=0;i<ArrayLen0(arr);i++) {
    if (Compare(arr[i],value) == 0) {
      return &arr[i];
    }
  }
  return 0;
}

#define Next TEMPLATE(Type,Next)
Type *Next(Type *arr,Type *value) {
  if (!arr) return 0;
  if (!value) return arr;
  if (value + 1 < arr + ArrayLen(arr)) {
    return value + 1;
  }
  return 0;
}

#define Prev TEMPLATE(Type,Prev)
Type *Prev(Type *arr,Type *value) {
  if (!arr) return 0;
  if (!value) return arr + ArrayLen(arr) - 1;
  if (value - 1 >= arr) {
    return value - 1;
  }
  return 0;
}

// FindClosestBinary - same as FindBinary but if no match returns closest
//   match, where element should be inserted before
// note: if element should be inserted at end, this is returning an element
// of the array that is actually invalid!
#define FindClosestBinary TEMPLATE(Type,FindClosestBinary)
Type *FindClosestBinary(Type *arr,Type value,int (*Compare)(Type,Type)) {
  unsigned mid,low=0,high=ArrayLen0(arr);

  if (!arr) return 0;
  while (low < high) {
    mid = low + ((high - low) / 2);
#ifdef ZZZ
fprintf(stderr,"compare: %s to %s ",arr[mid].key,value.key);
#endif
    if (Compare(arr[mid],value) < 0) {
#ifdef ZZZ
fprintf(stderr,"<\n");
#endif
      low = mid + 1;
    } else {
#ifdef ZZZ
fprintf(stderr,">=\n");
#endif
      high = mid;
    }
  }
  return &arr[low];
}

// FindBinary - binary search assuming data sorted by comparison function
#define FindBinary TEMPLATE(Type,FindBinary)
Type *FindBinary(Type *arr,Type value,int (*Compare)(Type,Type)) {
  unsigned mid,low=0,high=ArrayLen0(arr);
  while (low < high) {
    mid = (low + high) / 2;
    if (Compare(arr[mid],value) < 0) {
      low = mid + 1;
    } else {
      high = mid;
    }
  }
  if (low < ArrayLen0(arr) && !Compare(arr[low],value)) {
    return &arr[low];
  } else {
    return 0;
  }
}


// InsertSorted - add element to array, sorted into position
#define InsertSorted TEMPLATE(Type,InsertSorted)
int InsertSorted(Type **arr,Type entry,int (*Compare)(Type,Type)) {
  Type *before;
  int i;

  before = FindClosestBinary(arr[0],entry,Compare);
  i = before - arr[0];
  Before(arr,i,entry);
  return i;
}

#define ReplaceSorted TEMPLATE(Type,ReplaceSorted)
Type ReplaceSorted(Type **arr,Type entry,int (*Compare)(Type,Type)) {
  Type *before;
  int i;

  // Verify that FindClosestBinary actually returns the entry to replace if there is an exact match.
  before = FindClosestBinary(arr[0],entry,Compare);
  i = before - arr[0];
  if (arr[0] && i < ArrayLen(arr[0]) && Compare(*before,entry)==0) {
	  *before = entry;
  } else { 
	  Before(arr,i,entry);
  }
}

// Copy
// Sort
// ReplaceSorted - same as InsertSort, but if the entry already exists,
// it is replaced instead of a duplicate being inserted.  if an entry is
// replaced, it is returned, otherwise NULL is returned.
// using Find* to do a reverse search:
// suppose we have a connector number, and want to find the name
// the name:number assignment has the form of
//   attrib.Connector.Name.*=connectorNumber
// can't we just search for the number?  well we can, but the first
// part of the string also has to match!
/*
// a.val == connector # from attrib.Connector.<a.val>,<connpin>=<dionum>
*/

#undef Init0
#undef Before
#undef Push
#undef After
#undef Queue
#undef Delete
#undef Pop
#undef Pull
#undef Next
#undef Prev
#undef FindLinear
#undef FindBinary
#undef FindClosestBinary
#undef InsertSorted
#undef ReplaceSorted
/*
  This file is intended to replace array.h, but we have old code that
  uses array.h that we don't want to have to fix to use the new code.
  Author: Michael Schmidt (michael@embeddedARM.com)
 */
// Copyright (c) 2011, Technologic Systems, All Rights Reserved
// Refer to the COPYRIGHT file provided with this project for licensing terms.
